from setuptools import setup, find_packages

setup(
    name='wanglb_utils',  # 包名
    version='0.0.4',  # 版本
    description='常用函数方法汇总',  # 包简介
    long_description=open('README.md').read(),  # 读取文件中介绍包的详细内容
    include_package_data=True,  # 是否允许上传资源文件
    author='wanglb',  # 作者
    author_email='',  # 作者邮件
    maintainer='',  # 维护者
    maintainer_email='',  # 维护者邮件
    license='MIT License',  # 协议
    url='https://pypi.org/project/wanglb_utils/',  # github或者自己的网站地址
    packages=[
        'wanglb_utils'
    ],  # 包的目录
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',  # 设置编写时的python版本
    ],
    python_requires='>=3.7',  # 设置python版本要求
    install_requires=[
        'requests >= 2.0.0',
        'yaml>=6.0'
    ],  # 安装所需要的库
    entry_points={
        'console_scripts': [
            ''],
    },
)
